<?php

namespace App\Http\Controllers\Wallets;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use App\Http\Controllers\Controller;

use App\Models\Wallets\Wallet;

class WalletController extends Controller
{
   
    public function index(Request $request) {
        has_scope('wallets.read');
       
        try {
            $this->validate($request, [
                'latitude' => 'required',
                'longitude' => 'required',
            ]);
           
            // request parameters
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
            $lat   = $request->latitude;
            $long  = $request->longitude;

            // distance formula
            $haversineSQL = "(3956 * 2 * ASIN(SQRT( POWER(SIN(($lat-vendors.latitude) *  pi()/180 / 2), 2) +COS($lat * pi()/180) * COS(vendors.latitude * pi()/180) * POWER(SIN(($long - vendors.longitude) * pi()/180 / 2), 2)))*0.6213)";
            
            $obj    = Wallet::with('product.vendor','user');
             

            if(isset($request->order) && strtolower($request->order) == "asc") {
                $order  = $request->order;  
            }

            //user
            if($request->filled('customer_id')) { 
                $obj = $obj->where('user_id','=',$request->customer_id);
            }
            

            $result = $obj->leftjoin('products','products.id','=','wallets.product_id')
                          ->join('vendors','vendors.id','=','products.vendor_id')
                          ->select('wallets.*',DB::raw($haversineSQL . ' AS distance'))
                          ->orderBy('wallets.created_at', $order)
                          ->paginate($limit); 
    
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.WALLET_LIST'), 'records' => $result]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }

    }

    public function store (Request $request) {
        has_scope('wallets.write');
        
        $this->validate($request, [
           'product_id' => 'required|integer|exists:products,id',
           'user_id'    => 'required|integer|exists:customers,id',
        ]);

        try {

            $input = $request->all();
            $save  = Wallet::create($input);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.WALLET_ADDED'), 'records' => $save]);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function update (Request $request, $id) {
        has_scope('wallets.write');

        $wallet = Wallet::findOrFail($id);

        $this->validate($request, [
            'product_id' => 'required|integer|exists:products,id',
            'user_id'    => 'required|integer|exists:customers,id',
         ]);

        try {

            $input = $request->all();
            $save  = $wallet->update($input);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.WALLET_UPDATED'), 'records' => $save]);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function delete(Request $request, $id) {
        has_scope('wallets.delete');

        $wallet = wallet::findOrFail($id);

        try {

            $save  = $wallet->delete();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.WALLET_DELETED'), 'records' => $save]);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function count(Request $request) 
    {
        try {

            if($request->filled('customer_id')) {
                $count = Wallet::where('user_id',$request->customer_id)->count();
            } else {
                $count =  Wallet::count();
            }
           
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.WALLET_COUNT'), 'count' => $count]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function show($id) 
    {
        has_scope('wallets.read');

        $data = wallet::with('product','user')->findOrFail($id);

        try {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.WALLET_DETAIL'), 'result' => $data]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

}
