<?php

namespace App\Http\Controllers\v3\Vendors;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use App\Models\Vendors\Vendor;
use App\Models\Vendors\Address;
use Helpers;

class VendorAddressController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected $users;
    protected $user_address;
    protected  $user_id;
    protected  $role_fk;

    public function __construct(Vendor $users, Address $user_address, Request $request) 
    {
        $this->users = $users;
        $this->user_address = $user_address;

        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
        
        has_scope('vendors.read');
    }

    
    /**
     * @ user with its all address list api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function index(Request $request, $vendor_id) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if($limit > 500) {
                $limit = 25;
            }
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $id = $vendor_id;
            $users = $this->user_address::where([
                        ['vendor_id', '=', $id]
                    ])
                    ->where(function($q) use($request) {
                        // Global searching...
                        if(isset($request->search) && !empty($request->search) && $request->search!="") {
                            if(!empty($this->getColumnName())) {
                                foreach ($this->getColumnName() as $key => $col) {
                                    if($key==0) {
                                        $q->where($col, 'like', '%' .$request->search. '%');
                                    } else {
                                        $q->orWhere($col, 'like', '%' .$request->search. '%');
                                    }
                                }
                            }
                        }
                    })
                    ->where(function($q) use($request) {
                        // Column wise searching filter ...
                        $allowableColumn = $this->getColumnName();
                        if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                            $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                        }
                    })
                    ->orderBy($order_by_column_name, $order)
                    ->paginate($limit);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_DETAILS'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    /**
     * Add user address api.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request, $vendor_id)
    {
        has_scope('vendors.write');
        
        $this->validate($request, [
            'street_address' => 'required|max:255',
            'street_name' => 'required|max:255',
            'apartment_number' => 'required|numeric|min:1',
            'city' => 'required|max:255',
            'state' => 'required|max:255',
            'country' => 'required|max:255',
            'pincode' => 'required|digits_between:6,6',
            'landmark' => 'max:255',
            'location' => 'max:255',
            'country' => 'required|max:255',
        ]);

        try {

            $input = $request->all();
            $input['vendor_id'] = $vendor_id;
            $input['landmark'] = $request->landmark;
            $input['country'] = $request->country;
            //dd($input);
            $address = $this->user_address::create($input);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.ADDRESS_STORE'), 'records' => $address]);
        } 
        catch (Exception $e) {
            
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch(Throwable $e) {
            
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    
    /**
     * @ service List api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function show($vendor_id, $address_id) 
    {
        try {
            
            $address = $address = $this->user_address::where([
                ['id', '=', $address_id],
                ['vendor_id', '=', $vendor_id],
            ])->first();
            
            if( empty($address) ) {
                
                throw new Exception(config('service.USER_API.MESSAGE.ADDRESS_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => "Address Information!", 'records' => $address]);
        } 
        catch (Exception $e) {
            
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch(Throwable $e) {
            
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    
    /**
     * @ update user api .
     * @return \Illuminate\Http\Response
    */
    public function update(Request $request, $vendor_id, $address_id) {
        
        has_scope('vendors.write');
        
        $this->validate($request, [
            'street_address' => 'required|max:255',
            'street_name' => 'required|max:255',
            'apartment_number' => 'required|numeric|min:1',
            'city' => 'required|max:255',
            'state' => 'required|max:255',
            'pincode' => 'required|digits_between:6,6',
            'landmark' => 'max:255',
            'location' => 'max:255',
            'country' => 'required|max:255',
        ]);

        try {
            
            $address = $this->user_address::where([
                ['id', '=', $address_id],
                ['vendor_id', '=', $vendor_id]
            ]);
            
            if(!$address->update($request->all())) {
                
                throw new Exception(config('service.CONTRACTOR_API.MESSAGE.SOMETHING_WENT_WRONG'), 1);
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => "Address has been updated successfully!", 'records' => $address->first()]);
        } 
        catch (Exception $e) {
            
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch(Throwable $e) {
            
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    
    /**
     * @ set setDefaultAddress user api .
     * @return \Illuminate\Http\Response
    */
    public function setDefaultAddress(Request $request, $vendor_id, $address_id) {
        
        has_scope('vendors.write');
        
        try {
            
            $address = $this->user_address::where([
                ['id', '=', $address_id],
                ['vendor_id', '=', $vendor_id],
            ]);
            
            if (!$address->exists()) {
                
                throw new Exception(config('service.USER_API.MESSAGE.CUSTOMER_OR_ADDRESS_NOT_EXISTS'), 1);
            }
            
            // reset all
            $this->user_address::where('vendor_id', $vendor_id)->update(['is_default' => 0]);
            
            // set to default.
            $address->update(['is_default' => 1]);
            
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => "Default address has been updated successfully!", 'records' => $address->first()]);
                
        } 
        catch (Exception $e) {
            
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch(Throwable $e) {
            
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    
    /**
     * Remove the specified addres from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function delete($vendor_id, $address_id)
    {
        has_scope('vendors.delete');
        
        try {

            $address = $this->user_address::where([
                ['id', '=', $address_id],
                ['vendor_id', '=', $vendor_id],
            ])->delete();
            
            if(!$address) {
                
                throw new Exception(config('service.USER_API.MESSAGE.SOMETHING_WENT_WRONG'), 1);
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => "Address has been deleted successfully!", 'records' => $address]);
        } 
        catch (Exception $e) {
            
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch(Throwable $e) {
            
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $address = $this->user_address::first();
        if(!empty($address) && $address!="") {
            return array_keys($address->toArray());
        }

        return ['id'];
    }
}
