<?php

namespace App\Http\Controllers\v4\Enquiry;

use Exception;
use Throwable;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;

use App\Jobs\SendMailJob;

use App\Models\Enquiry\Enquiry;
use App\Models\Vendors\Vendor;

use App\Notifications\Vendors\Notification;

class EnquiryController extends Controller
{

    public function index(Request $request) {
        has_scope('enquiries.read');
       
        try {

            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;

            $obj  = Enquiry::with('vendor','user');

            if($request->filled('order') && strtolower($request->order) == "asc") {
            
                $order  = $request->order;  
            }
            
            //filtter by user or vendor
            if($request->filled('type')) {
                $obj = $obj->where('user_type', $request->type);

                if($request->filled('vendor_id')) {
                    $obj = $obj->where('vendor_id', $request->vendor_id);
                }
            }
            
            //filter by issue type
            if($request->filled('issue_type')) {
                $obj = $obj->where('issue_type', $request->issue_type);
            }

            //filtter by keyword
            if ($request->filled('keyword')) {
                $obj =  $obj->where(function ($query) use($request) {
                        $query->where('type', 'like', '%' . $request->keyword . '%')
                              ->orWhere('message', 'like', '%' . $request->keyword . '%');
                 });                 
            }
    
            $result = $obj->orderBy('created_at', $order)->paginate($limit);
    
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.ENQUIRY_LIST'), 'records' => $result]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }

    }

    public function store(Request $request) {
        has_scope('enquiries.write');

        $this->validate($request,[
         'title'    => 'required',
         'user_id'  => 'required',
         'issue_type' => 'required',
         'user_type'  => 'required|in:vendor,user',
         'message'    => 'required|string',
         'image'      => 'image|mimes:jpeg,png,jpg,gif,svg|max:2048',
        ]);

        try {
            $input = $request->all();

            if($request->user_type == 'user') {
                $input['user_id'] = $request->user_id;
            } else {
                unset($input['user_id']);
                $input['vendor_id'] = $request->user_id;
            }


            if ($request->has('image')) {
                $imageName = rand(10000, 9999999) . '-' . time() . '.' . request()->image->getClientOriginalExtension();
                request()->image->move(storage_path('enquiry'), $imageName);
                $input['image'] = $imageName;
            }
           
            $data  =  new Enquiry;
            $save  =  $data->create($input);

             //admin mail
             $greetingAdmin = 'Dear Admin';
             $bodyAdmin     =  $request->user_type == 'user' ? 'Someone contacted you. Please login to see the details' : ' A vendor raised a concern. Please login now to address';
             $typeAdmin     = 'New enquiry'; 
 
             $adminMail = new SendMailJob(env('ADMIN_EMAIL'), $greetingAdmin, $bodyAdmin, $typeAdmin);
             dispatch($adminMail);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.ENQUIRY_SUCCESS'), 'records' => $save]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function count() 
    {
        try {
            $count = Enquiry::count();
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.ENQUIRY_TOTAL'), 'count' => $count]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function show($id) 
    {
        has_scope('enquiries.read');

        try {
            $result = Enquiry::with('vendor','user')->findOrFail($id);
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.ENQUIRY_DETAIL'), 'records' => $result]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function delete($id) 
    {
        has_scope('enquiries.delete');

        try {
            $result = Enquiry::where('id',$id)->delete();
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.ENQUIRY_DELETE'), 'records' => $result]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function closemark($id) {
        has_scope('enquiries.write');

        $enquiry = Enquiry::findOrFail($id);

        try {

            $data = Enquiry::where('id',$id)->update(['status' => '1']);


            if($enquiry->user_type == 'vendor') {
                // $vendor = Vendor::select('first_name','last_name','email')->where('id',$enquiry->vendor_id)->first();

                //vendor mail
                // $greeting = 'Dear '.$vendor->first_name.' '.$vendor->last_name;
                // $body     = 'Your enquiry status has been changed.';
                // $type     = 'Enquiry Status Changed'; 

                // $mail = new SendMailJob($vendor->email, $greeting, $body, $type);
                // dispatch($mail);

                //vendor notification
                $vendor   = Vendor::find($enquiry->vendor_id);
                $greeting = 'Hi ' . $vendor->first_name . '!';
                $body     = 'Your enquiry status has been changed.';
                $type     =  'Enquiry Status Changed';

                $details = [
                    'greeting' => $greeting,
                    'body' => $body,
                    'type' => $type,
                    'notification_message' => 'Enquiry Status Changed.',
                    'thanks' => 'Regards Thanks',
                ];
                
                $role = '2';
                $vendor->notify(new Notification($details));
                $upd['role'] = $role;
                $notif_id = $vendor->notifications[0]->id;
                $vendor->notifications()->find($notif_id)->update($upd);

            }     
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.ENQUIRY_MARKED'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    } 
    
}
